;Tanks(!) program
;-------------------
;Binary created using DAsm 2.12 running on an Amiga.

   PROCESSOR   6502

BALLPHIY    EQU   #$0000
BALLPLOY    EQU   #$0001
BALLSCRPY   EQU   #$0700
BALLPHIX    EQU   #$0002
BALLPLOX    EQU   #$0003
BALLSCRPX   EQU   #$0703
BALLVLOY    EQU   #$0004
BALLVHIY    EQU   #$0005
BALLVLOX    EQU   #$0006
BALLVHIX    EQU   #$0007
BALLACCY    EQU   #$0008
BALLACCX    EQU   #$0009
BALLVISIBLE EQU   #$000A
FIRED       EQU   #$000B
HIT         EQU   #$000C
MISSED      EQU   #$000D
READY       EQU   #$000E
PLAYER      EQU   #$000F
CONTROLLER1 EQU   #$0010
CONTROLLER2 EQU   #$0011
TERRAINPTR  EQU   #$12        ;Pointer to Terrain
TERRCHRPTR  EQU   #$14        ;Pointer to Terrain CHR.
MAXTERRAIN  EQU   #$0016
TERRAINBASE EQU   #$0017      ;Pointer to start of Terrain.
P1RANDBASE  EQU   #$0018
P2RANDBASE  EQU   #$0019
PLANET      EQU   #$001A      ;Planet Number (0 is Earth...)
FADESTATUS  EQU   #$001B
;-----------------------
FREEZERO1   EQU   #$20        ;Free Zero-Page Pointers.
FREEZERO2   EQU   #$22
PALETTEPTR  EQU   #$24        ;Pointer to Current Palette.
;-----------------------
P1POSY      EQU   #$0040
P2POSY      EQU   #$0041
P1POSX      EQU   #$0042
P2POSX      EQU   #$0043
P1VINITHIY  EQU   #$0044
P2VINITHIY  EQU   #$0045
P1VINITLOY  EQU   #$0046
P2VINITLOY  EQU   #$0047
P1VINITHIX  EQU   #$0048
P2VINITHIX  EQU   #$0049
P1VINITLOX  EQU   #$004A
P2VINITLOX  EQU   #$004B


P1SCORE     EQU   #$004E
P2SCORE     EQU   #$004F
;-----------------------
VBWAIT      EQU   #$0050      ;Delays updating of game data
VBDELAY     EQU   #$0051      ;Timing for fades...
VBPASS      EQU   #$0052      ;Flag that VB has passed.
VBODD       EQU   #$0053      ;Flag for even/odd VBs.
MOVEMODE    EQU   #$0054
CRATERPOS   EQU   #$0055      ;Where to make crater
CRATERSIZE  EQU   #$0056      ;How big will it be?
EXPLDELAY   EQU   #$0057      ;Delay for explosions
PPU2000     EQU   #$0058      ;$2000 of PPU

;-------------------------------------------------------
GAMESCREEN  EQU   #$0059      ;ID for which game screen is running...
                              ;.. maybe also for what CHR-ROM in use...
                              ;0 = Tanks Screen
                              ;1 = Planets Screen
                              ;2 = Title Screen
                              ;3 = CMC Tech Screen
                              ;4 = Controls Screen
                              ;5 = Credits Screen
                              ;6 = Victory Screen
;---------------------------------------
PLANETCHANGE EQU  #$005A      ;Flag to change planets on selection screen.
ROUNDNUMBER EQU   #$005B      ;Round 1,2, or 3.
;----------------------

BALLCHR     EQU   #$0701
P1VHIY      EQU   #$0711         ;Sprites for Y,X Initial Velocity
P1VLOY      EQU   #$0715
P1VHIX      EQU   #$0719
P1VLOX      EQU   #$071D
P2VHIY      EQU   #$0721         ;Sprites for Y,X Initial Velocity
P2VLOY      EQU   #$0725
P2VHIX      EQU   #$0729
P2VLOX      EQU   #$072D
P1SCRSCORE  EQU   #$0731         ;Score on-screen
P2SCRSCORE  EQU   #$0735
P1CHRPOSY   EQU   #$0740
P1CHRPOSX   EQU   #$0743
P2CHRPOSY   EQU   #$0748
P2CHRPOSX   EQU   #$074B
EXPLSPR1    EQU   #$0751         ;Sprites for explosions
EXPLSPR2    EQU   #$0755         ;"       "
EXPLSPR3    EQU   #$0759         ;"       "
EXPLSPR4    EQU   #$075D         ;"       "

TEMPPAL     EQU   #$0600         ;Temporary storage for palette.
SOFTTERRAIN EQU   #$06BF         ;$700-64-1


   ORG   $C000    ;16Kb PRG-ROM, 8Kb CHR-ROM

Reset_Routine  SUBROUTINE
   cld         ;Clear decimal flag
   sei         ;Disable interrupts
;----------------------------------------------------------
   lda   $2002
   sta   P1RANDBASE        ;Randomize Stuff???
   lda   $4016
   sta   P2RANDBASE
   lda   #$00
   sta   TERRAINBASE

   lda   #$F0
   sta   MAXTERRAIN        ;1024 Bytes for now

;----------------------------------------------------------

.WaitV01   lda $2002
   bpl .WaitV01     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx PPU2000
   stx $2001      ;Screen display off, amongst other things
   dex
   txs         ;Top of stack at $1FF

;Clear (most of) the NES' WRAM. This routine is ripped from "Duck Hunt" - I should probably clear all
;$800 bytes.
   ldy #$07    ;To clear 8 x $100 bytes, from $000 to $7FF?
   sty $01        ;Store count value in $01
   ldy #$00
   sty $00
   lda #$00

.Clear   sta ($00),y    ;Clear $100 bytes
   dey
   bne .Clear

   dec $01        ;Decrement "banks" left counter
   bpl .Clear     ;Do next if >= 0

   jsr   WipePPU

;*******************************************************************
;-------------------------------------------------------------------
; PLANETS SCREEN
.PlanetsMain
   lda   #$01              ;Set CHR-ROM for mapper 3??
   sta   GAMESCREEN
   sta   $C000

   lda   #$00
   sta   P1SCORE
   sta   P2SCORE
   lda   #$01
   sta   ROUNDNUMBER
;-------------------------------------------------------------------
   lda   #$0B           ;Planets Palette
   jsr   SetPalette

   lda   #<.PlanetsMap
   sta   $00
   lda   #>.PlanetsMap
   sta   $01

   lda   #$20     ;Load Name & Attribute Table
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$00
   ldy   #$04

.LoadPlanets
   txa
   pha

   ldx   #$00
   lda   ($00),X  ;Load Planets Image
   sta   $2007

   pla
   tax

   inc   $00
   bne   .NoPlanets1
   inc   $01

.NoPlanets1
   dex
   bne   .LoadPlanets
   dey
   bne   .LoadPlanets



   ldx   #$00
   ldy   #$80              ;128 bytes for sprite data.
.CopyPlanetSprites
   lda   .PlanetsSpriteMap,X
   sta   $0700,X                ;$0700 is currently SPRITE MEMORY!!!!
   inx
   dey
   bne   .CopyPlanetSprites

   lda   #%10001000
   sta   $2000
   sta   PPU2000
   lda   #%00011110        ;Screen on, sprites on, show leftmost 8 pixels, colour
   sta   $2001

   lda   #$00
   sta   PLANET
   sta   VBWAIT
   lda   #$01
   sta   PLANETCHANGE

.PlanetsSelectLoop
   lda   VBWAIT
   bne   .PlanetsSelectLoop
   lda   CONTROLLER1
   and   #%00000010        ;Left
   beq   .NotDecreasePlanet
   lda   #%00000010
   eor   CONTROLLER1
   sta   CONTROLLER1
   lda   #$10
   sta   VBWAIT
   lda   #$01
   sta   PLANETCHANGE
   dec   PLANET
   bpl   .NotDecreasePlanet
   lda   #$09
   sta   PLANET

.NotDecreasePlanet
   lda   CONTROLLER1
   and   #%00000001        ;Right or Select
   beq   .NotIncreasePlanet
   lda   #%00000001
   eor   CONTROLLER1
   sta   CONTROLLER1
   lda   #$10
   sta   VBWAIT
   lda   #$01
   sta   PLANETCHANGE
   inc   PLANET
   lda   PLANET
   cmp   #$0A
   bne   .NotIncreasePlanet
   lda   #$00
   sta   PLANET

.NotIncreasePlanet
   lda   CONTROLLER1
   and   #%11010000
   beq   .NotSelectPlanet
   lda   #%11010000
   eor   CONTROLLER1
   sta   CONTROLLER1
   jmp   .GoToFight

.NotSelectPlanet
   ldx   PLANET
   lda   .PointerPosY,X       ;Load Pointer Y Position...
   sta   $077C                ;THIS MAY CHANGE!!!!
   lda   .PointerPosX,X       ;Load Pointer X Position
   sta   $077F                ;THIS MAY ALSO CHANGE!!!!

;----------------- RANDOMNESS --------------
   jsr   Randomness

   jmp   .PlanetsSelectLoop

.GoToFight
   lda   #$10
   sta   FADESTATUS     ;Do a fadeout
   lda   #$08
   sta   VBDELAY
.WaitPlanetFade
   lda   FADESTATUS
   bne   .WaitPlanetFade
.WaitV02   lda $2002
   bpl .WaitV02     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx PPU2000
   stx $2001      ;Screen display off, amongst other things
   jsr   WipePPU
   jsr   ClearSPRRAM
   jmp   .TanksMain
;   jmp   .VictoryMain

.PlanetsMap
   INCLUDE TanksPlanets.NAM

.PointerPosY
   dc.b  #$31,#$18,#$39,#$79,#$C2,#$DF,#$D1,#$A1,#$69,#$41

.PointerPosX
   dc.b  #$7C,#$96,#$B8,#$E2,#$C8,#$84,#$4B,#$23,#$18,#$3B

.PlanetsSpriteMap
   INCLUDE TanksPlanetsSPR.ASM

;-------------------------------------------------------

;*******************************************************************
;*******************************************************************
;                                                                  *
;    MAIN TANKS PROGRAM                                            *
;                                                                  *
;*******************************************************************
;*******************************************************************
;*******************************************************************
.TanksMain
   lda   #$00
   sta   $C000

   lda   PLANET
   jsr   SetPalette
;----------------- Draw Ground Patterns ---------------
.SetGround
   lda   #$20
   sta   $2006
   sta   $2006       ;Point to row 2.

   lda   #$02                 ;Counter to draw on both name tables
   sta   $0100
.DrawGroundLoop
   ldx   #$0D                 ;13 rows of 16x16 chars.
.DrawGroundLoop0 ldy   #$10   ;16 columns of 16x16 chars.
.DrawGroundLoop1 lda #$8C
   jsr   SetPlanetTerrain
   sta   $2007
   lda   #$8D
   jsr   SetPlanetTerrain
   sta   $2007
   dey
   bne   .DrawGroundLoop1
   ldy   #$10
.DrawGroundLoop2 lda #$8E
   jsr   SetPlanetTerrain
   sta   $2007
   lda   #$8F
   jsr   SetPlanetTerrain
   sta   $2007
   dey
   bne   .DrawGroundLoop2
   dex
   bne   .DrawGroundLoop0

   lda   #$24                 ;Name table 2
   sta   $2006
   lda   #$20
   sta   $2006
   dec   $0100
   bne   .DrawGroundLoop
;********* Set up Panel tiles *****************

.WaitVBPass1   lda   $2002       ;Wait 1 full VBlank (which seems to fix
   bpl   .WaitVBPass1            ;a nasty bug related to PPU writes....
.WaitVBPass2   lda   $2002
   bmi   .WaitVBPass2
.WaitVBPass3   lda   $2002       ;Hopefully I won't have to do much more of this.
   bpl   .WaitVBPass3


.SetPanelNAM
   ldx   #$23
   stx   $2006
   ldx   #$40
   stx   $2006

   ldx   #$00
   ldy   #$60     ;3 rows of Panel tiles
.InitPanel lda .PanelNAM,X
   sta   $2007
   inx
   dey
   bne   .InitPanel

   ldx   #$27
   stx   $2006
   ldx   #$40
   stx   $2006

   ldx   #$00
   ldy   #$60     ;3 rows of Panel tiles
.InitPanel2 lda .PanelNAM,X
   sta   $2007
   inx
   dey
   bne   .InitPanel2

   jmp   .SetAttMap
;**************************************************
.PanelNAM INCLUDE TanksPanelNAM.asm
;**************************************************
.PlanetsTerrGfx         ;What to add to standard gfx pointer
   dc.b  #$00,#$30,#$00,#$10,#$20,#$20,#$00,#$30,#$20,#$10
;************** SET ATTRIBUTE TABLES **************
.SetAttMap
   lda   #<.AttMap
   sta   FREEZERO1
   lda   #>.AttMap
   sta   FREEZERO1+1          ;Point to Palette

   ldx   PLANET
   beq   .NoAddAttMap
.AddAttMap
   lda   FREEZERO1
   clc
   adc   #$40                 ;Multiply PLANET by 64
   sta   FREEZERO1
   lda   FREEZERO1+1
   adc   #$00
   sta   FREEZERO1+1
   dex
   bne   .AddAttMap

.NoAddAttMap
   lda   #$23
   sta   $2006
   lda   #$C0
   sta   $2006

   ldy   #$00
   ldx   #$40     ;Set Attribute Map
.InitAtt lda   (FREEZERO1),Y
   sta $2007
   iny
   dex
   bne   .InitAtt

   lda   #$27
   sta   $2006
   lda   #$C0
   sta   $2006

   ldy   #$00
   ldx   #$40
.InitAtt2 lda  (FREEZERO1),Y
   sta   $2007
   iny
   dex
   bne   .InitAtt2
   jmp   .SetBallData
;*********************************************************
.AttMap
   INCLUDE TanksAtt.asm

.PlanetGravTable
   dc.b  #$06,#$03,#$04,#$0A,#$05,#$05,#$06,#$03,#$04,#$05
;*********************************************************
;-----------------BALL DATA ----------------------------
.SetBallData
;----------------- SET UP GRAVITY -------------
   ldx   PLANET
   lda   .PlanetGravTable,X
   sta   BALLACCY
;
   lda   #$00
   sta   FIRED
   sta   HIT
   sta   MISSED
   sta   PLAYER
;   sta   P1SCORE
;   sta   P2SCORE
   sta   MOVEMODE
   sta   VBPASS
   sta   VBODD

   lda   #$01
   sta   READY

   lda   #$30
   sta   P1POSX

   lda   #$D0
   sta   P2POSX


   lda   #$01
   sta   P1VINITHIY
   lda   #$00
   sta   P1VINITLOY

   lda   #$00
   sta   P1VINITHIX
   lda   #$40
   sta   P1VINITLOX

   lda   #$01
   sta   P2VINITHIY
   lda   #$00
   sta   P2VINITLOY

   lda   #$00
   sta   P2VINITHIX
   lda   #$40
   sta   P2VINITLOX

   lda   #$00
   sta   BALLACCX

   lda   #$01
   sta   BALLVISIBLE
   lda   #$02
   sta   BALLCHR


;------------------------------
   lda   #$D3       ;Set up Velocity Sprites
   sta   $0710
   sta   $0714
   lda   #$DC
   sta   $0718
   sta   $071C

   lda   #$1C
   sta   $0713
   sta   $071B
   lda   #$24
   sta   $0717
   sta   $071F

   lda   #$D3       ;Set up Velocity Sprites for P2
   sta   $0720
   sta   $0724
   lda   #$DC
   sta   $0728
   sta   $072C

   lda   #$DC
   sta   $0723
   sta   $072B
   lda   #$E4
   sta   $0727
   sta   $072F

   lda   #$D3        ;Set up Score Display
   sta   P1SCRSCORE-1
   sta   P2SCRSCORE-1
   lda   #$5D
   sta   P1SCRSCORE+2
   lda   #$9D
   sta   P2SCRSCORE+2

   lda   P1SCORE
   ora   #$10
   sta   P1SCRSCORE
   lda   P2SCORE
   ora   #$10
   sta   P2SCRSCORE

   lda   #$01
   sta   P1SCRSCORE+1     ;Set up palette
   sta   P2SCRSCORE+1
   sta   P1VHIY+1
   sta   P1VLOY+1
   sta   P1VHIX+1
   sta   P1VLOX+1
   sta   P2VHIY+1
   sta   P2VLOY+1
   sta   P2VHIX+1
   sta   P2VLOX+1


;-------------------------------

;Enable vblank interrupts, etc.
   lda   #%10001000
   sta   $2000
   sta   PPU2000

   jsr   InitTerrain
   jsr   InitTanks
   jsr   InitBall
   jsr   DrawTerrain

   ldx   #$00                       ;Put player 1 & 2 velocity on-screen.
   jsr   ScaleYVelocity
   jsr   ScaleXVelocity
   ldx   #$10
   jsr   ScaleYVelocity
   jsr   ScaleXVelocity
;-------------------------------------------
   lda   #$00
   sta   GAMESCREEN        ;Game Screen is "Tanks"(!)

.WaitV05   lda $2002
   bpl .WaitV05     ;Wait for vertical blanking interval

   lda   #%00011110        ;Screen on, sprites on, show leftmost 8 pixels, colour
   sta   $2001
;  cli            ;Enable interrupts(?)

   jsr   DisplayRound

;Now just loop forever?
.Loop

;---------------------------
   lda   FADESTATUS
   beq   .NoChangeRound
.WaitTanksFade
   jsr   Randomness
   lda   FADESTATUS
   bne   .WaitTanksFade
.WaitVTanksChange   lda $2002
   bpl .WaitVTanksChange     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx PPU2000
   stx $2001      ;Screen display off, amongst other things
   inc   ROUNDNUMBER
   lda   #$04
   cmp   ROUNDNUMBER
   beq   .NotLoopPlanets
   inc   PLANET
   lda   PLANET
   cmp   #$0A
   bne   .NotLoopPlanets
   lda   #$00
   sta   PLANET
.NotLoopPlanets
   jsr   WipePPU
   jsr   ClearSPRRAM
   lda   ROUNDNUMBER
   cmp   #$04
   bne   .NoVictory
   jmp   .VictoryMain
.NoVictory
   jmp   .TanksMain
.CompareRound2
;-----------------------------

.NoChangeRound

   lda   READY
   bne   .ReadyForControllerRead
   jmp   .NotReady

.ReadyForControllerRead

   lda   MOVEMODE
   beq   .NoTankMovement
   jmp   .NotDown
.NoTankMovement
;---------------------------------------------------------
   lda   PLAYER            ;Set up for Player 1 or 2
   asl
   asl
   asl
   asl
   tax
;----------------------------------------------------------

   lda   CONTROLLER1
   and   #%00001000
   beq   .NotUp
   lda   CONTROLLER1
   and   #%11110111
   sta   CONTROLLER1
   lda   BALLVLOY
   clc
   adc   #$01
   sta   BALLVLOY
   lda   BALLVHIY
   adc   #$00
   sta   BALLVHIY
;------------------- LIMIT VERTICAL SPEED -----------
   cmp   #$02
   bne   .NotTooHigh
   lda   #$01
   sta   BALLVHIY
   lda   #$FF
   sta   BALLVLOY
;----------------------------------------------------
.NotTooHigh
   jsr   ScaleYVelocity

.NotUp

   lda   CONTROLLER1
   and   #%00000100
   beq   .NotDown
   lda   CONTROLLER1
   and   #%11111011
   sta   CONTROLLER1
   lda   BALLVLOY
   sec
   sbc   #$01
   sta   BALLVLOY
   lda   BALLVHIY
   sbc   #$00
   sta   BALLVHIY
;------------------- LIMIT VERTICAL SPEED --------------
   cmp   #$00
   bne   .NotTooLow
   lda   BALLVLOY
   bne   .NotTooLow
   lda   #$01
   sta   BALLVLOY
;-------------------------------------------------------
.NotTooLow
   jsr   ScaleYVelocity

.NotDown

   lda   CONTROLLER1
   and   #%00000001              ;Right ->
   bne   .Right
   jmp   .NotRight
.Right
   lda   CONTROLLER1
   and   #%11111110
   sta   CONTROLLER1
;----------------------------
   lda   MOVEMODE
   bne   .CanMoveTankRight
   jmp   .NotMoveTankRight
.CanMoveTankRight
   lda   VBODD
   bne   .CanMoveTankRight2
   jmp   .NotRight

.CanMoveTankRight2
   txa
   pha
   ldx   PLAYER
   inc   P1POSX,X          ;Load player 1 OR 2 position??

   lda   P1POSX,X
   clc
   adc   #$08        ;Make level with middle of tank???
   lsr
   lsr
   lsr
   cpx   #$01
   bne   .NotPlayer2MoveR
   clc
   adc   #$20
.NotPlayer2MoveR
   tay                  ;Now Y contains X-column

   lda   P1POSX,X
   cmp   #$F1                 ;Is the tank at the right edge of the screen?
   bne   .SecondMoveTestR
   dec   P1POSX,X
   pla
   tax
   jmp   .NotRight

.SecondMoveTestR
   lda   (TERRAINPTR),Y
   cmp   #$03
   bpl   .ThirdMoveTestR
   dec   P1POSX,X
   pla
   tax
   jmp   .NotRight

.ThirdMoveTestR
   lda   (TERRAINPTR),Y
   sta   $0100             ;Right Tile
   tya
   pha
   lda   P1POSX,X
   clc
   adc   #$07              ;Test for middle of tank-1
   lsr
   lsr
   lsr
   cpx   #$01
   bne   .NotPlayer2MoveSecondR
   clc
   adc   #$20
.NotPlayer2MoveSecondR
   tay
   lda   (TERRAINPTR),Y
   sta   $0101
   pla
   tay
   inc   $0100             ;...Ignores gap of 1 space....
   lda   $0100
   sec
   sbc   $0101
   bmi   .TooBigOfAHill
   jmp   .MoveRightisFine

.TooBigOfAHill
   dec   P1POSX,X
   jmp   .NotRight

.MoveRightisFine
   dey
   tya
   asl
   asl
   asl
   sta   P1POSY,X

   clc
   lda   BALLPLOX             ;Copy tank position to ball position.
   adc   #$40
   sta   BALLPLOX
   lda   BALLPHIX
   adc   #$00
   sta   BALLPHIX

   jsr   PositionTanks
   jsr   PositionBall

   pla
   tax
   jmp   .NotRight
;----------------------------

.NotMoveTankRight
   lda   BALLVLOX
   clc
   adc   #$01
   sta   BALLVLOX
   lda   BALLVHIX
   adc   #$00
   sta   BALLVHIX
;----------------------- LIMIT HORIZONTAL SPEED --------
   cmp   #$03
   bne   .NotTooFarH
   lda   #$02
   sta   BALLVHIX
   lda   #$FF
   sta   BALLVLOX
;-------------------------------------------------------
.NotTooFarH
   jsr   ScaleXVelocity

.NotRight

   lda   CONTROLLER1
   and   #%00000010           ;Left
   bne   .Left
   jmp   .NotLeft
.Left
   lda   CONTROLLER1
   and   #%11111101
   sta   CONTROLLER1
;---------------------
   lda   MOVEMODE
   bne   .CanMoveTankLeft
   jmp   .NotMoveTankLeft
.CanMoveTankLeft
   lda   VBODD
   bne   .CanMoveTankLeft2
   jmp   .NotLeft

.CanMoveTankLeft2
   txa
   pha
   ldx   PLAYER
   dec   P1POSX,X       ;Load Player 1 or 2 position??

   lda   P1POSX,X
   clc
   adc   #$08           ;Make level with middle of tank
   lsr
   lsr
   lsr
   cpx   #$01
   bne   .NotPlayer2MoveL
   clc
   adc   #$20
.NotPlayer2MoveL
   tay

   lda   P1POSX,X
   cmp   #$00              ;Is it as left as it will go?
   bne   .SecondMoveTestL
   inc   P1POSX,X
   pla
   tax
   jmp   .NotLeft

.SecondMoveTestL
   lda   (TERRAINPTR),Y
   cmp   #$03
   bpl   .ThirdMoveTestL
   inc   P1POSX,X
   pla
   tax
   jmp   .NotLeft

.ThirdMoveTestL
   lda   (TERRAINPTR),Y
   sta   $0100
   tya
   pha
   lda   P1POSX,X
   clc
   adc   #$09        ;Test for middle of tank+1
   lsr
   lsr
   lsr
   cpx   #$01
   bne   .NotPlayer2MoveSecondL
   clc
   adc   #$20
.NotPlayer2MoveSecondL
   tay
   lda   (TERRAINPTR),Y
   sta   $0101
   pla
   tay
   inc   $0100
   lda   $0100
   sec
   sbc   $0101
   bmi   .TooBigOfAHillL
   jmp   .MoveLeftisFine

.TooBigOfAHillL
   inc   P1POSX,X
   jmp   .NotLeft

.MoveLeftisFine
   dey
   tya
   asl
   asl
   asl
   sta   P1POSY,X

   clc
   lda   BALLPLOX             ;Copy tank position to ball position.
   adc   #$40
   sta   BALLPLOX
   lda   BALLPHIX
   adc   #$00
   sta   BALLPHIX

   jsr   PositionTanks
   jsr   PositionBall

   pla
   tax
   jmp   .NotLeft
;---------------------
.NotMoveTankLeft

   lda   BALLVLOX
   sec
   sbc   #$01
   sta   BALLVLOX
   lda   BALLVHIX
   sbc   #$00
   sta   BALLVHIX
;----------------------- LIMIT HORIZONTAL SPEED --------
   cmp   #$FF
   bne   .NotTooLowH
   lda   BALLVLOX
   cmp   #$B8
   bne   .NotTooLowH
   lda   #$FF
   sta   BALLVHIX
   lda   #$B9
   sta   BALLVLOX
;-------------------------------------------------------
.NotTooLowH
   jsr   ScaleXVelocity


.NotLeft

   lda   MOVEMODE
   bne   .TestForBUp
   lda   CONTROLLER1
   and   #%01000000        ;B
   beq   .NotB
   lda   #$01          ;Enable tanks to move
   sta   MOVEMODE
   jmp   .NotB
.TestForBUp
   lda   CONTROLLER1
   and   #%01000000    ;When B button is pressed... for now..
   bne   .NotB
   lda   #$00
   sta   MOVEMODE

.NotB
   lda   CONTROLLER1
   and   #%10000000          ;A
   beq   .NotA

   ldx   PLAYER
   lda   #$01
   sta   FIRED
   lda   #$00
   sta   MISSED
   lda   #$00
   sta   READY
   lda   #$01
   sta   BALLVISIBLE
   lda   BALLVHIY             ;Copy Initial Velocities
   sta   P1VINITHIY,X
   lda   BALLVLOY
   sta   P1VINITLOY,X
   lda   BALLVHIX
   sta   P1VINITHIX,X
   lda   BALLVLOX
   sta   P1VINITLOX,X

   lda   PLAYER
   beq   .NotA
   lda   BALLVHIX
   eor   #$FF              ;Reverse direction if Player 2
   sta   BALLVHIX
   lda   BALLVLOX
   eor   #$FF
   sta   BALLVLOX

   lda   #$00
   sta   MOVEMODE

.NotA

.NotReady
   lda   VBPASS
   beq   .MainLoopEnd
   lda   #$00
   sta   VBPASS
   lda   FIRED
   beq   .MainLoopEnd
   jsr   Gravity            ;Gravity has been moved out of the VB code.
.MainLoopEnd
   jmp   .Loop

;*****************************************************
;------- VICTORY SCREEN ------------------------------

.VictoryMain

.WaitVictoryVB01   lda $2002
   bpl .WaitVictoryVB01     ;Wait for vertical blanking interval

   lda   #$06              ;Set CHR-ROM for mapper 3??
   sta   GAMESCREEN
   sta   $C000

   lda   #%10001000
   sta   PPU2000
   sta   $2000

   lda   #$0C
   jsr   SetPalette

   jsr   SetVictoryGradient

   lda   #$23
   sta   $2006
   lda   #$C0
   sta   $2006

   ldx   #$00
   ldy   #$40
.CopyVictoryAtt
   lda   .VictoryAtt,X
   sta   $2007
   inx
   dey
   bne   .CopyVictoryAtt

   lda   #$27
   sta   $2006
   lda   #$C0
   sta   $2006

   ldx   #$00
   ldy   #$40
.CopyVictoryAtt2
   lda   .VictoryAtt,X
   sta   $2007
   inx
   dey
   bne   .CopyVictoryAtt2

   lda   #$20
   sta   $2006
   lda   #$00
   sta   $2006
   jsr   VictoryDrawHighBars
   lda   #$24
   sta   $2006
   lda   #$00
   sta   $2006
   jsr   VictoryDrawHighBars

   lda   #$22
   sta   $2006
   lda   #$E0
   sta   $2006
   jsr   VictoryDrawLowBars
   lda   #$26
   sta   $2006
   lda   #$E0
   sta   $2006
   jsr   VictoryDrawLowBars
;---------------------------------
   lda   #<.VictoryTank
   sta   $00
   lda   #>.VictoryTank
   sta   $01

   lda   #$25     ;Load Name Table
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$A0
   ldy   #$01

.LoadVictoryTank
   txa
   pha

   ldx   #$00
   lda   ($00),X  ;Load Tank Image
   sta   $2007

   pla
   tax

   inc   $00
   bne   .NoVictoryTank01
   inc   $01

.NoVictoryTank01
   dex
   bne   .LoadVictoryTank
   dey
   bpl   .LoadVictoryTank
;-----------------------------------------------

   lda   #$00
   sta   $2005
   sta   $2005

   lda   #%10001000
   sta   PPU2000
   sta   $2000

   lda   #%00011110
   sta   $2001

   lda   #$00
   sta   $0002

   sta   $2005
   lda   #$00
   sta   $2005
.VictoryLoop
   jmp   .VictoryLoop


.VictoryTank
   INCLUDE TanksVictoryTankNAM.asm

.VictoryAtt
   INCLUDE TanksVictoryAtt.asm

VictoryDrawHighBars SUBROUTINE
   ldx   #$06
.HiBars01
   ldy   #$20
   lda   .VictoryTopBar,X
.HiBars02
   sta   $2007
   dey
   bne   .HiBars02
   dex
   bpl   .HiBars01
   rts

.VictoryTopBar dc.b #$06,#$05,#$04,#$05,#$04,#$03,#$03

VictoryDrawLowBars SUBROUTINE
   ldx   #$06
.LoBars01
   ldy   #$20
   lda   .VictoryBotBar,X
.LoBars02
   sta   $2007
   dey
   bne   .LoBars02
   dex
   bpl   .LoBars01
   rts

.VictoryBotBar dc.b #$03,#$03,#$FF,#$FE,#$FF,#$FE,#$FD

SetVictoryGradient SUBROUTINE

   ldx   #$3F
   stx   $2006
   ldx   #$05
   stx   $2006

   ldx   PLANET
   lda   .VictoryGradient,X
   sta   $2007

   ldx   #$3F
   stx   $2006
   ldx   #$09
   stx   $2006

   sta   $2007
   clc
   adc   #$10
   sta   $2007
   clc
   adc   #$10
   sta   $2007

   ldx   #$0D
   stx   $2007
   sta   $2007
   clc
   adc   #$10
   sta   $2007

   rts

.VictoryGradient dc.b #$01,#$00,#$06,#$07,#$07,#$0C,#$02,#$0D,#$03,#$08

DrawVictoryWinner SUBROUTINE
   lda   #$26
   sta   $2006
   lda   #$C5
   sta   $2006
   ldx   #$00
   ldy   #$16
.DrawVictory lda  .VictoryText,X
   sta   $2007
   inx
   dey
   bne   .DrawVictory

   lda   #$26
   sta   $2006
   lda   #$CC
   sta   $2006
   lda   P1SCORE
   cmp   P2SCORE
   bcs   .P1Wins
   lda   #$02
   jmp   .SetWinner
.P1Wins  lda   #$01
.SetWinner
   sta   $2007
   rts

.VictoryText dc.b #$F0,#$F1,#$F2,#$F3,#$F4,#$F5,#$00,#$00,#$00,#$F6,#$F7,#$F8,#$F9
   dc.b  #$00,#$FA,#$FB,#$F4,#$00,#$F6,#$F2,#$F5,#$FC

;*****************************************************
;-----------------------------------------------------
;*****************************************************

WipePPU SUBROUTINE
   lda   #$20
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$00
   ldy   #$10
.ClearPPU sta $2007
   dex
   bne   .ClearPPU
   dey
   bne   .ClearPPU

   lda   #$00
   sta   $2003
   tay
.ClearSpr sta   $2004
   dey
   bne   .ClearSpr
   rts

ClearSPRRAM SUBROUTINE
   pha
   txa
   pha
   tya
   pha

   lda   #$00
   tax
   tay
.ClearSPRRAM   sta   $0700,X        ;$700 FOR NOW!!!!
   inx
   dey
   bne   .ClearSPRRAM

   pla
   tay
   pla
   tax
   pla
   rts



SetPalette SUBROUTINE
;----------------------------------
; INPUT: A: PALETTE NUMBER
;----------------------------------
   tax
   lda   #<.Palette
   sta   PALETTEPTR
   lda   #>.Palette
   sta   PALETTEPTR+1          ;Point to Palette

   cpx   #$00
   beq   .NoAddPalette
.AddPalette
   lda   PALETTEPTR
   clc
   adc   #$20                 ;Multiply Palette Index by 32
   sta   PALETTEPTR
   lda   PALETTEPTR+1
   adc   #$00
   sta   PALETTEPTR+1
   dex
   bne   .AddPalette

.NoAddPalette
   ldx   #$3F
   stx   $2006
   ldx   #$00
   stx   $2006

   ldy   #$00
   ldx   #$20     ;Set BG & Sprite palettes.
.InitPal lda   (PALETTEPTR),Y
   sta   $2007
   sta   TEMPPAL,Y               ;Store to palette copy
   iny
   dex
   bne   .InitPal
   rts

;*********************************************************
.Palette
   INCLUDE  TanksPal.ASM
;*********************************************************

Randomness SUBROUTINE
   inc   P1RANDBASE
   lda   #$E1
   cmp   P1RANDBASE
   bne   .NoSplitTank1
   lda   #$01
   sta   P1RANDBASE
.NoSplitTank1
   lda   P2RANDBASE
   clc
   adc   #$03
   sta   P2RANDBASE
   lda   #$E1
   cmp   P2RANDBASE
   bcs   .NoSplitTank2
   lda   #$08
   sta   P2RANDBASE
.NoSplitTank2
   inc   TERRAINBASE              ;Set up random placement
   lda   #$F0                    ;1024 for now...
   cmp   TERRAINBASE
   bne   .NotOutofTerrain
   lda   #$00
   sta   TERRAINBASE
.NotOutofTerrain
   rts

ScaleYVelocity SUBROUTINE

   lda   BALLVLOY
   sta   P1VLOY,X
   lda   BALLVHIY
   sta   P1VHIY,X
   lsr   P1VHIY,X    ;Shift down 3 places (divide by 8)
   ror   P1VLOY,X
   lsr   P1VHIY,X
   ror   P1VLOY,X
   lsr   P1VHIY,X
   ror   P1VLOY,X

   lda   P1VLOY,X
   jsr   ToDecimal
   sta   P1VLOY,X

   lda   P1VLOY,X
   lsr
   lsr
   lsr
   lsr
   ora   #$10
   sta   P1VHIY,X     ;Fit into proper character space.
   lda   P1VLOY,X
   and   #$0F
   ora   #$10
   sta   P1VLOY,X
   rts

ScaleXVelocity SUBROUTINE

   lda   BALLVLOX
   sta   P1VLOX,X
   lda   BALLVHIX
   sta   P1VHIX,X

   lda   BALLVHIX
   cmp   #$FF
   bne   .NotInvertVelocity
   lda   #$00
   sta   P1VHIX,X
   lda   P1VLOX,X
   eor   #$FF           ;Invert Velocity
   sta   P1VLOX,X

.NotInvertVelocity

   lsr   P1VHIX,X    ;Shift down 3 places (divide by 8)
   ror   P1VLOX,X
   lsr   P1VHIX,X
   ror   P1VLOX,X
   lsr   P1VHIX,X
   ror   P1VLOX,X

   lda   BALLVHIX
   cmp   #$FF
   bne   .NotInvertVelocity2
   inc   P1VLOX,X

.NotInvertVelocity2

   lda   P1VLOX,X
   jsr   ToDecimal
   sta   P1VLOX,X

   lda   P1VLOX,X
   lsr
   lsr
   lsr
   lsr
   ora   #$10
   sta   P1VHIX,X     ;Fit into proper character space.
   lda   P1VLOX,X
   and   #$0F
   ora   #$10
   sta   P1VLOX,X

   lda   BALLVHIX
   cmp   #$FF
   bne   .NotInvertVelocity3
   lda   #$03
   sta   P1VHIX,X

.NotInvertVelocity3

   rts

ToDecimal SUBROUTINE
;--------------------------------------------
; INPUT: A: Number to convert from Hex to Dec
;--------------------------------------------
   sta   $0100
   tya
   pha

   lda   $0100
   ldy   #$FF
.ToDecLoop1
   iny
   sec
   sbc   #$0A
   bcs   .ToDecLoop1
   clc
   adc   #$0A
   sta   $0100
   tya
   asl
   asl
   asl
   asl
   ora   $0100
   sta   $0100

   pla
   tay
   lda   $0100
   rts

SetPlanetTerrain SUBROUTINE            ;Set proper appearance for terrain on planets.
   sta   $0102
   txa
   pha

   ldx   PLANET
   lda   .PlanetTerrainTable,X
   clc
   adc   $0102
   sta   $0102

   pla
   tax
   lda   $0102
   rts

.PlanetTerrainTable
   dc.b  #$00,#$30,#$40,#$10,#$40,#$20,#$50,#$30,#$20,#$10

DrawTerrain SUBROUTINE
;--------------------------------------
; INPUTS: A: Name Table Location (??)
;         Y: Pointer Counter
;--------------------------------------
   lda   PPU2000
   eor   #%00000100
   sta   $2000
   sta   PPU2000
   ldx   #$00
   stx   $0100             ;Temporary
   lda   #$20
   sta   $0101
   ldy   #$00
.DrawTerr1
   lda   $0101
   sta   $2006
   lda   $0100
   sta   $2006
   lda   (TERRAINPTR),Y    ;Must use Y reg to get proper value!!
   tax
   lda   #$03
.DrawTerr2
   sta   $2007
   dex
   bne   .DrawTerr2
   lda   #$1A              ;30 Rows(?) - 3 rows of Menu
   sec
   sbc   (TERRAINPTR),Y
   tax
   lda   (TERRCHRPTR),Y
;   lda   #$80
   jsr   SetPlanetTerrain
   sta   $2007
   dex
.NotHill
;   lda   #$80
.DrawTerr3
;   sta   $2007
   dex
   bne   .DrawTerr3
   inc   $0100
   iny
   lda   #$20
   and   $0100
   beq   .DrawTerr1
   lda   #$00
   sta   $0100
   inc   $0101
   inc   $0101
   inc   $0101
   inc   $0101
   cpy   #$40
   bne   .DrawTerr1
   lda   PPU2000
   eor   #%00000100        ;Inc Address by 1
   sta   $2000
   sta   PPU2000
   rts

InitTerrain SUBROUTINE
;   lda   #$F0              ;For now, 1024-64 bytes (/4)
;   sta   MAXTERRAIN
   lda   #<.TerrainMap
   sta   TERRAINPTR
   lda   #>.TerrainMap
   sta   TERRAINPTR+1

   lda   #$00
   sta   $0100
   lda   TERRAINBASE
   asl
   rol   $0100
   asl
   rol   $0100
   sta   $0101
   clc
   adc   TERRAINPTR
   sta   TERRAINPTR
   lda   $0100
   adc   TERRAINPTR+1
   sta   TERRAINPTR+1

   lda   #<.TerrainCharMap
   sta   TERRCHRPTR
   lda   #>.TerrainCharMap
   sta   TERRCHRPTR+1

   lda   $0101
   clc
   adc   TERRCHRPTR
   sta   TERRCHRPTR
   lda   $0100
   adc   TERRCHRPTR+1
   sta   TERRCHRPTR+1

;--------------------- COPY HARD COPY OF TERRAIN TO MEMORY  (??) ----
   ldy   #$00
   ldx   #$40
.CopyTerrain   lda   (TERRAINPTR),Y
   sta   SOFTTERRAIN,Y
   iny
   dex
   bne   .CopyTerrain
   lda   #<SOFTTERRAIN        ;Point to memory copy of terrain.
   sta   TERRAINPTR
   lda   #>SOFTTERRAIN
   sta   TERRAINPTR+1
;---------------------------------------------------------------------
   rts

.TerrainMap
   INCLUDE Terrain1024.asm

.TerrainCharMap
   INCLUDE TanksTerrainChar.asm

;***************************************************

InitTanks SUBROUTINE

.InitP1X
   lda   P1RANDBASE
   and   #%11111000
   sta   P1POSX
   lda   P2RANDBASE
   and   #%11111000
   sta   P2POSX
;----------- Get Y-Position from Terrain ------------
   lda   P1POSX
   lsr
   lsr
   lsr
   tay
   lda   (TERRAINPTR),Y
   tay
   cmp   #$03
   bpl   .X1isFine
   lda   P1POSX
   clc
   adc   #$08
   sta   P1POSX
   jmp   .InitP1X

.X1isFine
   dey
   tya
   asl
   asl
   asl
   sta   P1POSY

.InitP2X
   lda   P2POSX
   lsr
   lsr
   lsr
   clc
   adc   #$20
   tay
   lda   (TERRAINPTR),Y
   tay
   cmp   #$03
   bpl   .X2IsFine
   lda   P2POSX
   clc
   adc   #$08
   sta   P2POSX
   jmp   .InitP2X

.X2IsFine
   dey
   tya
   asl
   asl
   asl
   sta   P2POSY
;----------------------------------------------------
.SetUpTanksPos
   jsr   PositionTanks
   rts

PositionTanks SUBROUTINE
   pha
   txa
   pha

;--------------- Alter Tank Position ------------
   lda   P1POSX
   lsr
   lsr
   lsr
   tay
   iny
;------------------------
   lda   (TERRAINPTR),Y
   tax
   dex
   txa
   asl
   asl
   asl
   sta   P1POSY
;------------------------
   ldx   #$00           ;Player1
   jsr   GetHeight
;------------------------
   lda   P2POSX
   lsr
   lsr
   lsr
   clc
   adc   #$20
   tay
   iny
;------------------------
   lda   (TERRAINPTR),Y
   tax
   dex
   txa
   asl
   asl
   asl
   sta   P2POSY
;--------------------------
   ldx   #$01           ;Player2
   jsr   GetHeight
;-------------------------------

   lda   P1POSY            ;Set up Tank sprites & Position
   sta   P1CHRPOSY
   sta   P1CHRPOSY+4
   lda   P1POSX
   sta   P1CHRPOSX
   lda   #$08
   clc
   adc   P1POSX
   sta   P1CHRPOSX+4
   lda   #$20
   ldx   #$01
   sta   P1CHRPOSY+1
   lda   #$21
   inx
   sta   P1CHRPOSX+2

   lda   P2POSY
   sta   P2CHRPOSY
   sta   P2CHRPOSY+4
   lda   #%01000000
   sta   P2CHRPOSY+2
   sta   P2CHRPOSY+6
   lda   P2POSX
   sta   P2CHRPOSX
   lda   #$08
   clc
   adc   P2POSX
   sta   P2CHRPOSX+4
   lda   #$21
   sta   P2CHRPOSY+1
   lda   #$20
   sta   P2CHRPOSX+2

   pla
   tax
   pla
   rts

GetHeight SUBROUTINE
;--------------------------------------
; IN: Y: Pointer along Terrain
;     X: Player
;--------------------------------------
   lda   (TERRCHRPTR),Y
   and   #$0F              ;Reduce to 0-F
   tay
   dey
   bne   .NotSteepRising   ;Get height from character graphics
   lda   P1POSX,X          ;And from tank's position
   and   #$07
   sta   $0100
   lda   #$08
   sec
   sbc   $0100
   jmp   .HeightGiven
.NotSteepRising
   dey
   bne   .NotSteepFalling
   lda   P1POSX,X
   and   #$07
   clc
   adc   #$01
   jmp   .HeightGiven
.NotSteepFalling
   dey
   bne   .NotLowRising
   lda   P1POSX,X
   and   #$07
   lsr
   sta   $0100
   lda   #$08
   sec
   sbc   $0100
   jmp   .HeightGiven
.NotLowRising
   dey
   bne   .NotHighRising
   lda   P1POSX,X
   and   #$07
   lsr
   sta   $0100
   lda   #$04
   sec
   sbc   $0100
   jmp   .HeightGiven
.NotHighRising
   dey
   bne   .NotHighFalling
   lda   P1POSX,X
   and   #$07
   lsr
   clc
   adc   #$01
   jmp   .HeightGiven
.NotHighFalling
   dey
   bne   .NotLowFalling
   lda   P1POSX,X
   and   #$07
   lsr
   clc
   adc   #$05
   jmp   .HeightGiven
.NotLowFalling
   dey
   bne   .NotRoughTerrain
   lda   #$01
   jmp   .HeightGiven
.NotRoughTerrain
   dey
   bne   .NotHillTop
   lda   P1POSX,X
   and   #$07
   sta   $0100
   and   #%00000100
   bne   .SubHillTop
   lda   $0100
   and   #%00000011
   asl
   sta   $0100
   dec   $0100
   lda   #$07
   sec
   sbc   $0100
   jmp   .HeightGiven
.SubHillTop
   lda   $0100
   and   #%00000011
   asl
   clc
   adc   #$01
   jmp   .HeightGiven
.NotHillTop
   lda   #$00
.HeightGiven
   clc
   adc   P1POSY,X
   sta   P1POSY,X
   rts

ShowHideTanks SUBROUTINE
   lda   BALLPHIX
   and   #$40
   beq   .HideTank2
   lda   #$00
   sta   P1CHRPOSY+1
   sta   P1CHRPOSY+5
   lda   #$21
   sta   P2CHRPOSY+1
   lda   #$20
   sta   P2CHRPOSY+5
   jmp   .SaveTanks
.HideTank2
   lda   #$00
   sta   P2CHRPOSY+1
   sta   P2CHRPOSY+5
   lda   #$20
   sta   P1CHRPOSY+1
   lda   #$21
   sta   P1CHRPOSY+5

.SaveTanks
   rts

DisplayRound SUBROUTINE
   lda   #%10001000
   sta   $2000
   lda   #$00
   sta   $2005
   sta   $2005
   lda   #$23
   sta   $2006
   lda   #$89
   sta   $2006
   ldx   #$00
.DrawRound lda  .RoundText,X
   beq   .NoDrawRound
   sta   $2007
   inx
   jmp   .DrawRound
.NoDrawRound
   lda   #$23
   sta   $2006
   lda   #$8F
   sta   $2006
   lda   ROUNDNUMBER
   ora   #$30
   sta   $2007
   lda   PPU2000
   sta   $2000
   rts

.RoundText dc.b "Round   FIGHT!"
   dc.b #$00

DisplayMissed SUBROUTINE
   lda   #$40
   sta   VBWAIT
   lda   #$23
   sta   $2006
   lda   #$89
   sta   $2006
   ldx   #$00
.DrawMiss1 lda  .MissText,X
   beq   .NoDrawMiss1
   sta   $2007
   inx
   jmp   .DrawMiss1
.NoDrawMiss1

   lda   #$27
   sta   $2006
   lda   #$89
   sta   $2006
   ldx   #$00
.DrawMiss2 lda  .MissText,X
   beq   .NoDrawMiss2
   sta   $2007
   inx
   jmp   .DrawMiss2
.NoDrawMiss2

   lda   P1SCORE
   ora   #$10
   sta   P1SCRSCORE
   lda   P2SCORE
   ora   #$10
   sta   P2SCRSCORE
   rts

.MissText   dc.b "     MISS!    "
   dc.b  #$00

DisplayHit SUBROUTINE
   lda   #$60
   sta   VBWAIT
   lda   #$23
   sta   $2006
   lda   #$89
   sta   $2006
   ldx   #$00
.DrawHit1 lda  .HitText,X
   beq   .NoDrawHit1
   sta   $2007
   inx
   jmp   .DrawHit1
.NoDrawHit1

   lda   #$27
   sta   $2006
   lda   #$89
   sta   $2006
   ldx   #$00
.DrawHit2 lda  .HitText,X
   beq   .NoDrawHit2
   sta   $2007
   inx
   jmp   .DrawHit2
.NoDrawHit2

   lda   P1SCORE
   ora   #$10
   sta   P1SCRSCORE
   lda   P2SCORE
   ora   #$10
   sta   P2SCRSCORE
   rts

.HitText    dc.b "     HIT!     "
   dc.b #$00

ClearMessage SUBROUTINE
   lda   #$23
   sta   $2006
   lda   #$89
   sta   $2006
   lda   #$00
   ldy   #$0E
.ClearMsg1
   sta   $2007
   dey
   bne   .ClearMsg1

   lda   #$27
   sta   $2006
   lda   #$8E
   sta   $2006
   lda   #$00
   ldy   #$05
.ClearMsg2
   sta   $2007
   dey
   bne   .ClearMsg2
;----------------------------
;----------------------------
   lda   ROUNDNUMBER
   cmp   #$01
   bne   .CompareRound2
   lda   #$03
   cmp   P1SCORE
   beq   .CloseRound
   lda   #$03
   cmp   P2SCORE
   beq   .CloseRound
   jmp   .NoScoreTest
.CompareRound2
   lda   ROUNDNUMBER
   cmp   #$02
   bne   .CompareRound3
   lda   #$06
   cmp   P1SCORE
   beq   .CloseRound
   lda   #$06
   cmp   P2SCORE
   beq   .CloseRound
   jmp   .NoScoreTest
.CompareRound3
   lda   ROUNDNUMBER
   cmp   #$03
   bne   .NoScoreTest
   lda   #$09
   cmp   P1SCORE
   beq   .CloseRound
   lda   #$09
   cmp   P2SCORE
   beq   .CloseRound
   jmp   .NoScoreTest
.CloseRound
   lda   #$10
   sta   FADESTATUS     ;Do a fadeout
   lda   #$08
   sta   VBDELAY
.NoScoreTest


   rts

InitBall SUBROUTINE

   jsr   PositionBall

   ldx   PLAYER
   lda   P1VINITHIY,X
   sta   BALLVHIY
   lda   P1VINITLOY,X
   sta   BALLVLOY

   lda   P1VINITHIX,X
   sta   BALLVHIX
   lda   P1VINITLOX,X
   sta   BALLVLOX

   rts

PositionBall SUBROUTINE
   pha
   txa
   pha
   tya
   pha
   ldx   PLAYER
   lda   P1POSY,X
   sec
   sbc   #$04           ;Make ball appear higher than tank.
   eor   #$FF
   sta   BALLPLOY
   lda   #$00
   sta   BALLPHIY
   asl   BALLPLOY
   rol   BALLPHIY
   asl   BALLPLOY
   rol   BALLPHIY
   asl   BALLPLOY
   rol   BALLPHIY
   asl   BALLPLOY
   rol   BALLPHIY
   asl   BALLPLOY
   rol   BALLPHIY
   asl   BALLPLOY
   rol   BALLPHIY

   ldx   PLAYER
   lda   P1POSX,X
   clc
   adc   #$08
   sta   BALLPLOX
   lda   #$00
   sta   BALLPHIX
   asl   BALLPLOX
   rol   BALLPHIX
   asl   BALLPLOX
   rol   BALLPHIX
   asl   BALLPLOX
   rol   BALLPHIX
   asl   BALLPLOX
   rol   BALLPHIX
   asl   BALLPLOX
   rol   BALLPHIX
   asl   BALLPLOX
   rol   BALLPHIX

   lda   PLAYER
   beq   .NotPlayer2InitBall
   clc
   lda   BALLPHIX
   adc   #$40
   sta   BALLPHIX
.NotPlayer2InitBall

   pla
   tay
   pla
   tax
   pla
   rts

Gravity SUBROUTINE
   lda   BALLVLOY
   sec
   sbc   BALLACCY
   sta   BALLVLOY
   bcs   .NoOverflowVY
   dec   BALLVHIY
.NoOverflowVY

   lda   BALLPLOY
   clc
   adc   BALLVLOY
   sta   BALLPLOY
   bcc   .NoOverflowPLOY
   inc   BALLPHIY
.NoOverflowPLOY
   lda   BALLPHIY
   clc
   adc   BALLVHIY
   sta   BALLPHIY

   lda   BALLPHIY
   pha
   lda   BALLPLOY
   pha
   lsr   BALLPHIY
   ror   BALLPLOY
   lsr   BALLPHIY
   ror   BALLPLOY
   lsr   BALLPHIY
   ror   BALLPLOY
   lsr   BALLPHIY
   ror   BALLPLOY
   lsr   BALLPHIY
   ror   BALLPLOY
   lsr   BALLPHIY
   ror   BALLPLOY
   lda   BALLPLOY
   eor   #$FF
   sta   BALLSCRPY

   pla
   sta   BALLPLOY
   pla
   sta   BALLPHIY

;---------------------- X-POS -------------
   lda   BALLVLOX
   clc
   adc   BALLACCX
   sta   BALLVLOX
   bcc   .NoOverflowVX
   inc   BALLVHIX
.NoOverflowVX

   lda   BALLPLOX
   clc
   adc   BALLVLOX
   sta   BALLPLOX
   bcc   .NoOverflowP1LOX
   inc   BALLPHIX
.NoOverflowP1LOX
   lda   BALLVHIX
   clc
   adc   BALLPHIX
   sta   BALLPHIX

.ShiftPX
   lda   BALLPHIX
   pha
   lda   BALLPLOX
   pha
   lsr   BALLPHIX
   ror   BALLPLOX
   lsr   BALLPHIX
   ror   BALLPLOX
   lsr   BALLPHIX
   ror   BALLPLOX
   lsr   BALLPHIX
   ror   BALLPLOX
   lsr   BALLPHIX
   ror   BALLPLOX
   lsr   BALLPHIX
   ror   BALLPLOX
   lda   BALLPLOX
   sta   BALLSCRPX

   pla
   sta   BALLPLOX
   pla
   sta   BALLPHIX

;------------ TESTING FOR OFF-SCREEN ------------------------------
   lda   BALLPHIY
   and   #%11000000
   beq   .StillVisible
   lda   #$00
   jmp   .GravEnd
.StillVisible
   lda   #$01
.GravEnd
   sta   BALLVISIBLE

;------------ TEST FOR MISS -------------
   lda   FIRED
   bne   .YesHitDetect
   jmp   .NoHitDetect
.YesHitDetect
   lda   BALLVISIBLE
   beq   .NotYMissed
   lda   #$DF
   cmp   BALLSCRPY
   bcs   .NotYMissed

   lda   #$01
   sta   MISSED
   lda   #$00
   sta   FIRED
   lda   #$00
   sta   BALLVISIBLE

.NotYMissed
   lda   BALLPHIX
   and   #$80
   beq   .NotXMissed
   lda   #$01
   sta   MISSED
   lda   #$00
   sta   FIRED
   lda   #$00
   sta   BALLVISIBLE
.NotXMissed

;---------- TEST FOR GROUND HIT ------------------

   lda   FIRED
   bne   .YesGroundDetect
   jmp   .NoHitDetect
.YesGroundDetect
   lda   BALLVISIBLE
   beq   .NotGroundHit

   lda   BALLSCRPX      ;X-Position
   lsr
   lsr
   lsr
   tay
   lda   BALLPHIX
   lsr
   and   #$20
   sta   $0100          ;Is it on Screen 1 or 2?
   tya
   clc
   adc   $0100
   tay
   lda   BALLSCRPY      ;Y-Value
   lsr
   lsr
   lsr
   cmp   (TERRAINPTR),Y
   bpl   .GroundHit
   jmp   .NotGroundHit
.GroundHit
;----------- Make Craters -------------------
   lda   #$06           ;Is it Neptune??
   cmp   PLANET
   bne   .CreateCrater
;Do neptunian things
   jmp   .CraterMade
.CreateCrater
   ldx   #$00
   jsr   Explosion            ;Create Explosion Sprites
   dey
   sty   CRATERPOS
   lda   #$03
   sta   CRATERSIZE
;--------------------------------------------
.CraterMade
   lda   #$01
   sta   MISSED
   lda   #$00
   sta   FIRED
   lda   #$00
   sta   BALLVISIBLE

.NotGroundHit

;----- TEST FOR HIT ------------------------------
   lda   PLAYER               ;Is Player 1 Firing?
   bne   .Player2HitTest
   lda   BALLVISIBLE          ;Is the ball on(the proper)screen?
   beq   .NoP1HitTest
   lda   BALLPHIX             ;Is the ball in the 2nd Player's Screen?
   and   #%01000000           ;(Pos=40-7F)
   beq   .NoP1HitTest
   lda   BALLSCRPY
   sec
   sbc   P2POSY               ;Is it at or above Y-pos?
   bmi   .NoP1HitTest
   sec
   sbc   #$08
   bpl   .NoP1HitTest
   lda   BALLSCRPX
   sec
   sbc   P2POSX               ;Is it at or above X-pos?
   bmi   .NoHitTestYet
   sec
   sbc   #$10                 ;Is it within 16 pixels? (Size of Tank)
   bpl   .NoHitTestYet

.P1IsHit
   ldx   #$01
   jsr   Explosion
   lda   #$01
   sta   HIT
   lda   #$00
   sta   FIRED
   lda   #$00
   sta   BALLVISIBLE
.NoP1HitTest   jmp   .NoHitTestYet

.Player2HitTest
   lda   BALLVISIBLE          ;Is the ball on(the proper)screen?
   beq   .NoHitTestYet
   lda   BALLPHIX             ;Is the ball in the 1st Player's Screen?
   and   #%01000000           ;(Pos=40-7F)
   bne   .NoHitTestYet
   lda   BALLSCRPY
   sec
   sbc   P1POSY               ;Is it at or above Y-pos?
   bmi   .NoHitTestYet
   sec
   sbc   #$08
   bpl   .NoHitTestYet
   lda   BALLSCRPX
   sec
   sbc   P1POSX               ;Is it at or above X-pos?
   bmi   .NoHitTestYet
   sec
   sbc   #$10                 ;Is it within 16 pixels? (Size of Tank)
   bpl   .NoHitTestYet

.P2IsHit
   ldx   #$01
   jsr   Explosion
   lda   #$01
   sta   HIT
   lda   #$00
   sta   FIRED
   lda   #$00
   sta   BALLVISIBLE

.NoHitTestYet
;------------------------------------
.NoHitDetect
   rts


;***************************************************

DisplayPlanetName SUBROUTINE
   ldx   #$00
   ldy   PLANET
   beq   .NoMultPlanetName
   lda   #$00
.MultPlanetName
   clc
   adc   #$07
   dey
   bne   .MultPlanetName
   tax
.NoMultPlanetName

   lda   #$21                 ;Center (about) planet name.
   sta   $2006
   lda   #$8D
   sta   $2006

   ldy   #$07           ;Length of name
.WritePlanetName
   lda   .PlanetNames,X
   sta   $2007
   inx
   dey
   bne   .WritePlanetName
   rts

.PlanetNames
   dc.b " Earth "
   dc.b "  Moon "
   dc.b "  Mars "
   dc.b "Jupiter"
   dc.b " Saturn"
   dc.b " Uranus"
   dc.b "Neptune"
   dc.b " Pluto "
   dc.b "Mercury"
   dc.b " Venus "

Fadeout SUBROUTINE
   lda   #$3F
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$00
   stx   $0100
   ldy   #$20
.FadePal
   ldx   $0100
   lda   TEMPPAL,X
   tax
   lda   .FadeIndex,X   ;Get Fade info.
   sta   $2007
   ldx   $0100
   sta   TEMPPAL,X
   inc   $0100
   dey
   bne   .FadePal

   dec   FADESTATUS
   lda   #$00
   sta   $2005
   sta   $2005
   lda   #%10001000
   ora   PPU2000
   sta   PPU2000
   sta   $2000
   lda   #%00011110
   sta   $2001

   rts

.FadeIndex
   INCLUDE TanksFadePal.ASM

Explosion SUBROUTINE
   pha
   tya
   pha
   lda   BALLSCRPY         ;Get ball position
   sec
   sbc   #$04
   sta   EXPLSPR2-1        ;Save to explosion Y position.
   sta   EXPLSPR1-1
   sta   EXPLSPR4-1
   clc
   adc   #$08
   sta   EXPLSPR3-1
   lda   #$01
   sta   EXPLSPR2+1        ;Save attributes
   sta   EXPLSPR3+1
   sta   EXPLSPR1+1
   sta   EXPLSPR4+1
   lda   BALLSCRPX
   sec
   sbc   #$04
   sta   EXPLSPR2+2        ;Save to explosion X position
   sta   EXPLSPR3+2
   sec
   sbc   #$08
   sta   EXPLSPR1+2
   clc
   adc   #$10
   sta   EXPLSPR4+2
   lda   #$2E
   sta   EXPLSPR1
   lda   #$30
   sta   EXPLSPR2
   lda   #$2F
   sta   EXPLSPR3
   lda   #$2E
   sta   EXPLSPR4
   cpx   #$01
   bne   .RegularExplosion
;-------------------------
   lda   EXPLSPR1+2
   clc
   adc   #$05
   sta   EXPLSPR1+2
   lda   EXPLSPR3-1
   sec
   sbc   #$06
   sta   EXPLSPR3-1
   lda   EXPLSPR4+2
   sec
   sbc   #$05
   sta   EXPLSPR4+2

.RegularExplosion
;----------------
   lda   EXPLSPR1+2       ;Is it off-screen????
   cmp   #$F8
   bcc   .NoExpl1OffScreen
   lda   #$00
   sta   EXPLSPR1+2
.NoExpl1OffScreen
   lda   EXPLSPR4+2       ;Is it off-screen????
   cmp   #$07
   bcs   .NoExpl4OffScreen
   lda   #$F8
   sta   EXPLSPR4+2
.NoExpl4OffScreen
   lda   EXPLSPR3-1       ;Is it too low?
   cmp   #$C8
   bcc   .NoExpl3OffScreen
   lda   #$C8
   sta   EXPLSPR3-1
.NoExpl3OffScreen
   lda   #$04
   sta   EXPLDELAY
   pla
   tay
   pla
   rts

MakeCrater SUBROUTINE
   tya
   pha
   cmp   #$FF
   beq   .NoLCrater        ;Is it off the map?
   tya
   cmp   #$40              ;Is it off the map?
   beq   .NoLCrater
   lda   #$01              ;Is it too high?
   cmp   (TERRAINPTR),Y
   beq   .NoLCrater
   lda   #$19
   cmp   (TERRAINPTR),Y    ;Is it too low?
   beq   .NoLCrater
   jsr   DrawCrater        ;Draw Crater space
   lda   (TERRAINPTR),Y
   clc
   adc   #$01              ;Increase Terrain depth
   sta   (TERRAINPTR),Y
.NoLCrater
   pla
   tay
   rts

DrawCrater SUBROUTINE
;------------------------------------
; INPUTS: Y = Centre of hit
;------------------------------------
   tya
   and   #%00100000
   bne   .CraterOnScreen2
   lda   #$20
   sta   $0100
   jmp   .CalcPPUPos
.CraterOnScreen2
   lda   #$24
   sta   $0100
.CalcPPUPos
   lda   (TERRAINPTR),Y
   tax
   lda   #$00
   sta   $0101
.Add32 clc
   lda   $0101
   adc   #$20
   sta   $0101
   lda   $0100
   adc   #$00
   sta   $0100
   dex
   bne   .Add32

   tya
   and   #%00011111     ;Strip 32 off of Y
   clc
   adc   $0101
   sta   $0101
   lda   $0100
   adc   #$00
   sta   $0100

   lda   $0100          ;Point to proper row
   sta   $2006
   lda   $0101
   sta   $2006

   lda   PPU2000
   eor   #%00000100
   sta   $2000
   sta   PPU2000
;-------------------------------------------------------
   lda   #$03
   sta   $2007              ;Fill hole with blank chars
   lda   #$89
   jsr   SetPlanetTerrain
   sta   $2007          ;And then with a hilly char.
;-------------------------------------------------------

   lda   PPU2000
   eor   #%00000100
   sta   $2000
   sta   PPU2000
   rts


NMI_Routine SUBROUTINE
   php
   pha
   txa
   pha
   tya
   pha

   lda   #$01
   sta   VBPASS
   lda   VBODD
   eor   #$01
   sta   VBODD

;---------- READ JOYPAD -----------------------

   ldx   #$01
   stx   $4016
   stx   $4017
   dex
   stx   $4016
   stx   $4017
   ldy   #$08     ;Read 8 bits from $4016
   lda   #$00
   sta   CONTROLLER1
   ldx   PLAYER
.ReadCont1
   lda   $4016            ;,X for Controller 2.
   ror
   lda   CONTROLLER1
   rol
   sta   CONTROLLER1
   dey
   bne   .ReadCont1
   sta   CONTROLLER1
;---------------------------------------------
;--- TEST FOR FADEOUT ------------------------
   lda   FADESTATUS
   beq   .NoFade
   lda   VBDELAY
   bne   .DelayFade
   jsr   Fadeout
   lda   #$04
   sta   VBDELAY
   jmp   .NoFade
.DelayFade
   dec   VBDELAY
;---------------------------------------------
.NoFade


   lda   GAMESCREEN
   cmp   #$01
   bne   .NotGameScreenPlanets
   lda   VBWAIT
   beq   .NoPlanetChangeWait
   dec   VBWAIT
.NoPlanetChangeWait
   lda   PLANETCHANGE
   beq   .PlanetChangeEnd
   lda   #$00
   sta   PLANETCHANGE

   jsr   DisplayPlanetName

.PlanetChangeEnd
   lda   #$00
   sta   $2005
   sta   $2005

   jmp   .VBEnd
.NotGameScreenPlanets
   cmp   #$06
   bne   .NotGameScreenVictory
   lda   $0002
   cmp   #$FF
   bne   .VictoryIncreaseScroll
   lda   #$FF
   jmp   .VictorySetScroll
.VictoryIncreaseScroll
   inc   $0002
   lda   $0002
   cmp   #$FF
   bne   .VictorySetScroll
   jsr   DrawVictoryWinner
   lda   #$FF
.VictorySetScroll
   sta   $2005
   lda   #$00
   sta   $2005
   lda   #%10001000
   sta   $2000
   jmp   .VBEnd
.NotGameScreenVictory

;---- Update Explosions --------
   lda   EXPLSPR4          ;Has the explosion finished?
   beq   .ExplEnd
   lda   EXPLDELAY           ;Has the proper delay elapsed?
   bne   .ExplEnd
   lda   EXPLSPR4
   cmp   #$36
   beq   .ClearExplosion
   inc   EXPLSPR2
   inc   EXPLSPR3
   inc   EXPLSPR1
   inc   EXPLSPR4
   lda   #$04
   sta   EXPLDELAY
   jmp   .ExplEnd
.ClearExplosion
   lda   #$00
   sta   EXPLSPR2
   sta   EXPLSPR3
   sta   EXPLSPR1
   sta   EXPLSPR4
   lda   #$04
   sta   EXPLDELAY
.ExplEnd
   dec   EXPLDELAY

;------------------- DRAW CRATER, IF ANY --------------
   lda   CRATERSIZE
   beq   .DontMakeCrater
   ldy   CRATERPOS
   jsr   MakeCrater
   lda   #$02
   cmp   CRATERSIZE
   bne   .NoDeepCrater
   jsr   MakeCrater
.NoDeepCrater
   dec   CRATERSIZE
   inc   CRATERPOS
;   jsr   PositionTanks
   lda   #$00
   sta   $2005
   sta   $2005
.DontMakeCrater

   lda   VBWAIT
   beq   .NoWaiting
   dec   VBWAIT               ;Once VBWait clear, switch players.
   bne   .NotFinishedMsg
   eor   PLAYER               ;Switch between Player1 and Player2
   sta   PLAYER
   jsr   PositionTanks
   jsr   InitBall
   jsr   ClearMessage
   lda   #$01
   sta   READY
   jmp   .NoWaiting
.NotFinishedMsg
   jmp   .VBEnd
.NoWaiting

   lda   MISSED
   bne   .Missed
   lda   HIT
   bne   .Hit
   lda   FIRED
   beq   .NotFired
   jmp   .UpdateSprites
.Hit
   lda   #$00
   sta   HIT
   ldx   PLAYER
   inc   P1SCORE,X
   jsr   DisplayHit
   jmp   .UpdateSprites
.Missed
   jsr   DisplayMissed
   lda   #$00
   sta   MISSED
.NotFired
.UpdateSprites
   lda   BALLVISIBLE
   asl
   sta   BALLCHR

   jsr   ShowHideTanks

;   lda   #$07
;   sta   $4014             ;Copy $700 to Spr-RAM

;--- SET SCROLLING ----------
   lda   #$40
   and   BALLPHIX
   beq   .ScreenA
   lda   #%10001001
   jmp   .SaveScrolling
.ScreenA lda   #%10001000
.SaveScrolling
   sta   $2000
   sta   PPU2000
   lda   #$00
   sta   $2005
   sta   $2005

.VBEnd
   lda   #$07
   sta   $4014 
   pla
   tay
   pla
   tax
   pla
   plp
   rti

IRQ_Routine       ;Dummy label
   rti

;That's all the code. Now we just need to set the vector table approriately.

   ORG   $FFFA,0
   dc.w  NMI_Routine
   dc.w  Reset_Routine
   dc.w  IRQ_Routine    ;Not used, just points to RTI


;The end.
